#include <iostream>
#include "canvas.h"
#include "data.h"
#include <QtGui>

using namespace std;

Canvas::Canvas(QWidget* p) : QWidget(p) {
	setPalette(Qt::darkGray);
	setAutoFillBackground(true);
}

QSize Canvas::minimumSizeHint() const {
	return QSize(200,200);
}

QSize Canvas::sizeHint() const {
	return QSize(200,200);
}

void Canvas::paintEvent(QPaintEvent*) {
	Model* model = Data::currModel();
	if(model) {
		//cerr	<< "Painting a model with " << model->nrObj() << " object(s) "
		//		<< "and " << model->nrTimeP() << " time point(s).\n";

		// Create painter
		QPainter painter(this);
		painter.setRenderHint(QPainter::Antialiasing);
		painter.setPen(Qt::black);

		// Set painter coordinates
		int mmx = model->minx();
		int mmy = model->miny();
		painter.setWindow(model->minx(),model->miny(),model->width(),model->height());

		// TODO: vang problemen op met modellen van hoogte of breedte 0.

		double strx = double(width()) / double(model->width());
		double stry = double(height()) / double(model->height());
		bool tr = strx < stry;
		int nh = int(model->height() * strx);
		int nw = int(model->width() * stry);
		if(tr) painter.setViewport(0,(height()-nh)/2,width(),nh); 
		else painter.setViewport((width()-nw)/2,0,nw,height()); 
		painter.fillRect(model->minx(),model->miny(),model->width(),model->height(),Qt::white);

		int t = Data::currTime();
		for(MIVOI it = model->dEnd(t); it != model->dBegin(t);) {
			--it;
			for(int m = 0; m < (it->second).size(); ++m) {
				painter.save();
				QRect rect = paintObj((it->second)[m],t,painter);
				rect = QRect(rect.x()-mmx, rect.y()-mmy, rect.width(),rect.height());
				//cerr << "Draw text at rectangle (" << rect.x() << ',' << rect.y() << ',' << rect.width() << ',' << rect.height() << ").\n";
				if(tr) rect = QRect(rect.x() * strx,rect.y() * strx + ((height()-nh)/2),rect.width() * strx,rect.height() * strx);
				else rect = QRect(rect.x() * stry + ((width()-nw)/2),rect.y() * stry,rect.width() * stry,rect.height() * stry);
				QPainter tpainter(this);
				tpainter.setFont(Data::font());
				tpainter.setPen(((it->second)[m])->pencolor(t));
				tpainter.drawText(rect,Qt::AlignCenter,(((it->second)[m])->text(t)).c_str());

				painter.restore();
			}
		}

	}
	else paintNoModel();
}

void Canvas::paintNoModel() {
	QString nomod("No model loaded");

	QFont font("arial",20);
	font.setBold(true);
	QFontMetrics fm(font);
	int w = fm.width(nomod);
	int h = fm.height();

	QPainter painter(this);
	painter.setFont(font);
	painter.drawText((width()-w)/2,(height()-h)/2,nomod);
}

QRect Canvas::paintObj(Object* obj, int t, QPainter& painter) {
	if(obj->shape(t)) {
		//cerr << "We paint the object ";
		//for(int n = 0; n < (obj->name()).size(); ++n) cerr << (obj->name())[n] << ' ';
		//cerr << "at timepoint " << t << " and depth " << obj->depth(t) << ".\n";

		painter.translate(obj->xpos(t),obj->ypos(t));
		painter.setBrush(obj->color(t));
		painter.setPen(obj->pencolor(t));
	
		switch(obj->shape(t)->type()) {
			case POLY:
				paintPolyObj(obj->shape(t),painter);
				break;
			case ELLI:
				paintElliObj(obj->shape(t),painter);
				break;
			default:
				cerr << "A fatal error occurred in Canvas::paintObj.\n";
		}

		QTransform matr = painter.worldTransform();
//		matr = matr.inverted();
		return matr.mapRect(obj->brect(t));
	}
	else return QRect(0,0,100,12);
}

void Canvas::paintPolyObj(Shape* s, QPainter& painter) {
	PolyShape* ps = dynamic_cast<PolyShape*>(s);
	QPoint points[ps->nrPoints()];
	for(int n =0; n < ps->nrPoints(); ++n) points[n] = ps->point(n);
	painter.drawPolygon(points,ps->nrPoints());
}

void Canvas::paintElliObj(Shape* s, QPainter& painter) {
	ElliShape* el = dynamic_cast<ElliShape*>(s);
	painter.drawEllipse(0,0,el->width(),el->height());
}

