#include <iostream>
#include "model.h"

/************
	Objects 
************/

Object::Object(const vector<string>& n) : _name(n), _shape(0), _xpos(0), _ypos(0), _depth(50), _color(QColor(255,255,255,0)), _pencolor(0,0,0), _text("") { }

/** Mutators **/

void Object::set_shape(Shape* s, int t) {
	while(_dynshape.size() <= t) _dynshape.push_back(0);
	_dynshape[t] = s;
}

void Object::set_xpos(int x, int t) {
	while(_dynxpos.size() <= t) _dynxpos.push_back(0);
	_dynxpos[t] = x;
}

void Object::set_ypos(int y, int t) {
	while(_dynypos.size() <= t) _dynypos.push_back(0);
	_dynypos[t] = y;
}

void Object::set_text(const string& tx, int t) {
	while(_dyntext.size() <= t) _dyntext.push_back("");
	_dyntext[t] = tx;
}

void Object::set_depth(int d, int t) {
	while(_dyndepth.size() <= t) _dyndepth.push_back(50);
	_dyndepth[t] = d;
}

void Object::set_color(int r, int g, int b) {
	_color = QColor(r,g,b);
}

void Object::set_pencolor(int r, int g, int b) {
	_pencolor = QColor(r,g,b);
}

void Object::set_color(int r, int g, int b, int t) {
	while(_dyncolor.size() <= t) _dyncolor.push_back(QColor(255,255,255,0));
	_dyncolor[t] = QColor(r,g,b);
}

void Object::set_pencolor(int r, int g, int b, int t) {
	while(_dynpencolor.size() <= t) _dynpencolor.push_back(QColor(255,255,255,0));
	_dynpencolor[t] = QColor(r,g,b);
}

/** Inspectors **/

Shape* Object::shape(int t) const {
	if(_dynshape.empty()) return _shape;
	else if(_dynshape.size() > t) return _dynshape[t];
	else return 0;
}

int Object::xpos(int t) const {
	if(_dynxpos.empty()) return _xpos;
	else if(_dynxpos.size() > t) return _dynxpos[t];
	else return 0;
}

int Object::ypos(int t) const {
	if(_dynypos.empty()) return _ypos;
	else if(_dynypos.size() > t) return _dynypos[t];
	else return 0;
}

string Object::text(int t) const {
	if(_dyntext.empty()) return _text;
	else if(_dyntext.size() > t) return _dyntext[t];
	else return "";
}

int Object::depth(int t) const {
	if(_dyndepth.empty()) return _depth;
	else if(_dyndepth.size() > t) return _dyndepth[t];
	else return 50;
}

QColor Object::color(int t) const {
	if(_dyncolor.empty()) return _color;
	else if(_dyncolor.size() > t) return _dyncolor[t];
	else return QColor(255,255,255,0);
}

QColor Object::pencolor(int t) const {
	if(_dynpencolor.empty()) return _pencolor;
	else if(_dynpencolor.size() > t) return _dynpencolor[t];
	else return QColor(255,255,255,0);
}

int Object::maxTime() const {
	int m = 0;
	m = (_dynshape.size() > m ? _dynshape.size() : m);
	m = (_dynxpos.size() > m ? _dynxpos.size() : m);
	m = (_dynypos.size() > m ? _dynypos.size() : m);
	m = (_dyndepth.size() > m ? _dyndepth.size() : m);
	m = (_dyncolor.size() > m ? _dyncolor.size() : m);
	m = (_dynpencolor.size() > m ? _dynpencolor.size() : m);
	m = (_dyntext.size() > m ? _dyntext.size() : m);
	return m-1;
}

int Object::minx() const {
	int r = minx(0);
	for(int t = 1; t <= maxTime(); ++t) {
		int n = minx(t);
		r = (r < n) ? r : n;
	}
	return r;
}

int Object::miny() const {
	int r = miny(0);
	for(int t = 1; t <= maxTime(); ++t) {
		int n = miny(t);
		r = (r < n) ? r : n;
	}
	return r;
}

int Object::maxx() const {
	int r = maxx(0);
	for(int t = 1; t <= maxTime(); ++t) {
		int n = maxx(t);
		r = (r > n) ? r : n;
	}
	return r;
}

int Object::maxy() const {
	int r = maxy(0);
	for(int t = 1; t <= maxTime(); ++t) {
		int n = maxy(t);
		r = (r > n) ? r : n;
	}
	return r;
}

int Object::minx(int t) const {
	Shape* s = shape(t);
	if(s) return (s->minx() + xpos(t));
	else return 0;
}

int Object::miny(int t) const {
	Shape* s = shape(t);
	if(s) return (s->miny() + ypos(t));
	else return 0;
}

int Object::maxx(int t) const {
	Shape* s = shape(t);
	if(s) return (s->maxx() + xpos(t));
	else return 0;
}

int Object::maxy(int t) const {
	Shape* s = shape(t);
	if(s) return (s->maxy() + ypos(t));
	else return 0;
}

QRect Object::brect(int t) const {
	Shape* s = shape(t);
	if(s) return s->brect();
	else return QRect(0,0,100,12);
}

/***********
	Models 
***********/

Object* Model::object(const vector<string>& name) const {
	map<vector<string>,Object*>::const_iterator i = _objects.find(name);
	if(i == _objects.end()) return 0;
	else return i->second;
}

void Model::analyze() {
	map<vector<string>,Object*>::iterator it = _objects.begin();

	// 1. Calculate the number of timepoints
	int maxtime = 0;
	for(; it != _objects.end(); ++it) {
		int mt = it->second->maxTime();
		maxtime = (mt > maxtime ? mt : maxtime);
	}
	_sortedobj = vector<map<int,vector<Object*> > >(maxtime+1);

	// 2. Calculate the minimum and maximum needed width and heigth
	it = _objects.begin();
	if(it != _objects.end()) {
		_minx = it->second->minx();
		_miny = it->second->miny();
		_maxx = it->second->maxx();
		_maxy = it->second->maxy();
		++it;
	}
	//cerr << "Upper left corner: (" << _minx << "," << _miny << ").\n";
	//cerr << "Lower right corner: (" << _maxx << "," << _maxy << ").\n";

	for(; it != _objects.end(); ++it) {
		int ix = it->second->minx();	_minx = (ix < _minx ? ix : _minx);
		int iy = it->second->miny();   _miny = (iy < _miny ? iy : _miny);
		int ax = it->second->maxx();   _maxx = (ax > _maxx ? ax : _maxx);
		int ay = it->second->maxy();   _maxy = (ay > _maxy ? ay : _maxy);
	//	cerr << "Upper left corner: (" << _minx << "," << _miny << ").\n";
	//	cerr << "Lower right corner: (" << _maxx << "," << _maxy << ").\n";
	}


	// 3. Per timepoint, sort objects according to depth
	for(int n = 0; n <= maxtime; ++n) {
		it = _objects.begin();
		for(; it != _objects.end(); ++it) {
			Object* obj = it->second;
			(_sortedobj[n][obj->depth(n)]).push_back(obj);
		}
	}
}
