#ifndef MODEL_H
#define MODEL_H

#include <QColor>
#include <vector>
#include <map>
#include <string>
#include "shape.h"

using namespace std;

/************
	Objects
************/


class Object {
	
	protected:
		vector<string>		_name;

		Shape*				_shape;
		int					_xpos;
		int					_ypos;
		int					_depth;
		QColor				_color;
		QColor				_pencolor;
		string				_text;

		vector<Shape*>		_dynshape;
		vector<int>			_dynxpos;
		vector<int>			_dynypos;
		vector<int>			_dyndepth;
		vector<QColor>		_dyncolor;
		vector<QColor>		_dynpencolor;
		vector<string>		_dyntext;

	public:

		// Constructors
		Object(const vector<string>& n);

		// Mutators
		void	set_shape(Shape* s)			{ _shape = s;	}
		void	set_xpos(int x)				{ _xpos = x;	}	
		void	set_ypos(int y)				{ _ypos = y;	}
		void	set_text(const string& t)	{ _text = t;	}
		void	set_depth(int d)				{ _depth = d;	}
		void	set_color(int,int,int);
		void	set_pencolor(int,int,int);

		void	set_shape(Shape*,int);
		void	set_xpos(int,int);		
		void	set_ypos(int,int);		
		void	set_text(const string&,int);
		void	set_depth(int,int);
		void	set_color(int,int,int,int);
		void	set_pencolor(int,int,int,int);

		// Inspectors
		vector<string>	name()			const { return _name;	}
		Shape*			shape(int)		const;
		int				xpos(int)		const;
		int				ypos(int)		const;
		string			text(int)		const;
		int				depth(int)		const;
		QColor			color(int)		const;
		QColor			pencolor(int)	const;
		int				minx()			const;
		int				miny()			const;
		int				maxx()			const;
		int				maxy()			const;
		int				minx(int)		const;
		int				miny(int)		const;
		int				maxx(int)		const;
		int				maxy(int)		const;
		int				maxTime()		const;
		QRect				brect(int)		const;
	
};


/***********
	Models
***********/

typedef map<vector<string>,Object*>					MVSO;
typedef map<vector<string>,Object*>::iterator	MVSOI;
typedef map<int,vector<Object*> >					MIVO;
typedef map<int,vector<Object*> >::iterator		MIVOI;
typedef vector<map<int,vector<Object*> > >		VMIVO;

class Model {

	private:
		MVSO	_objects;
		int	_minx;
		int	_miny;
		int	_maxx;
		int	_maxy;
		VMIVO	_sortedobj;
		
	public:

		// Inspectors
		int		minx()								const { return _minx;					}
		int		miny()								const { return _miny;					}
		int		maxx()								const { return _maxx;					}
		int		maxy()								const { return _maxy;					}
		int		width()								const { return _maxx - _minx;			}
		int		height()								const { return _maxy - _miny;			}
		int		nrObj()								const { return _objects.size();		}
		int		nrTimeP()							const { return _sortedobj.size();	}
		int		maxTime()							const { return _sortedobj.size()-1;	}
		Object*	object(const vector<string>&)	const;

		MIVOI		dBegin(int t)								{ return _sortedobj[t].begin();	}
		MIVOI		dEnd(int t)									{ return _sortedobj[t].end();		}

		// Mutators
		void	addObj(Object* o) { _objects[o->name()] = o;	}
		void	analyze();
};

#endif
