"""
ASPCCGTK library

Copyright (C) 2011, 2012 Yuliya Lierler
Copyright (C) 2011, 2012 Peter Schueller

requires python >= 2.6
"""


import sys
from bioasp import asp
import time

def is_number(s):
  try:
    int(s)
    return True
  except ValueError:
    return False 

def debug(s):
  import inspect
  print >>sys.stderr, "DBG %03d: %s" % \
    (inspect.currentframe().f_back.f_lineno,s)

def process_commandline(options):
  '''
  process commandline arguments
  modifies given dictionary
  may exit program and output error or help message
  '''
  debug('starting commandline processing with options ' + repr(options))
  assert(options.__class__ == dict)
  i=1
  while i<len(sys.argv):    
    debug('processing commandline argument ' + repr(sys.argv[i]))
    if is_number(sys.argv[i]):
      options['maxstep_options'] = '--const maxsteps='+sys.argv[i]
    elif  sys.argv[i].find("-p=")!=-1:
      options['parser_asp']=sys.argv[i][sys.argv[i].find("-p=")+3:]
    elif  sys.argv[i].find("-folio")!=-1:
      options['clasp_bin']='./claspfoliowrapper.sh'
    elif  sys.argv[i].find("-nd")!=-1:
      options['no_display']=True
    elif  sys.argv[i].find("-d=")!=-1:
      options['dictionary_asp']=sys.argv[i][sys.argv[i].find("-d=")+3:]
    elif  sys.argv[i].find("-i=")!=-1:
      options['idpdraw_asp']=sys.argv[i][sys.argv[i].find("-i=")+3:]
    elif  sys.argv[i].find("-numtr=")!=-1:
      options['numtr_options'] = '--const numtr='+sys.argv[i][sys.argv[i].find("-numtr=")+7:]
    elif  sys.argv[i].find("-stats")!=-1:
      options['stats'] = True
    elif  sys.argv[i].find("-strict")!=-1:
      options['only_strict'] = True
    elif  sys.argv[i].find("-no-normalize")!=-1:
      options['normalize'] = False
    elif  sys.argv[i].find("-use-features")!=-1:
      options['use_features'] = True
    elif  sys.argv[i].find("-disambiguate")!=-1:
      options['disambiguat'] = True
    elif  sys.argv[i].find('-h')!=-1:
      print "OPTIONS:"
      print "-p=<ccg.asp> -- specifies parser encoding to use"
      print "-d=<dictionary.asp> -- specifies parser encoding to use"
      print "-i=<ccg2idpdraw.asp> -- specifies encoding to use for"
      print "    converting parser answer sets to idpdraw answer sets"
      print "integer -- specifies maxsteps"
      print "-numtr=<integer> -- specifies numtr constant (number of allowed type raising rules) in configuration encoding"
      print "<instance_file> -- specifies instance to parse"
      print "-nd -- disables displaying of parse trees"
      print "-folio -- invokes claspfolio"
      print "-stats -- retrieve statistics from gringo and clasp and display some of them"
      print "-strict -- only do strict parsing (no optimization)"
      print "-no-normalize -- do not apply CCG normalization techniques"
      print "-disambiguate -- use semantic_disambiguation.asp to disambiguate parse trees"
      print "-h -- prints these options"

      sys.exit()
    else:
      options['input_asp'] = sys.argv[i]
    i=i+1
  debug('updated options ' + repr(options))

def analyze_instance(options):
  ret = {
    'maxprobclass' : None
  }
  # first get set of tag probability classes by "solving" the input program without parser
  solver_preinfo = asp.GringoClasp(
    clasp_bin=options['clasp_bin'],
    gringo_bin=options['gringo_bin'])
  preinfo_model = solver_preinfo.run(
    [options['input_asp']],
    additionalProgramText="""
      #hide.
      maxprobclass(X) :- X = #max [ tag_probability_class(Y,Z) = Y ].
      noprobclass :- maxprobclass(#infimum).
      probclass(X) :- not noprobclass, maxprobclass(X).
      #show probclass/1.
      maxpos(X) :- X = #max [ word_at(W,Y) = Y ].
      #show maxpos/1.
      """,
    collapseAtoms=False,collapseTerms=False)
  debug(preinfo_model)
  assert(len(preinfo_model) == 1)
  for atom in preinfo_model[0]:
    if atom.pred() == 'probclass':
      ret['maxprobclass'] = atom.arg(0)
    if atom.pred() == 'maxpos':
      ret['maxpos'] = atom.arg(0)
  #debug('analysis result ' + repr(ret))
  return ret

def get_parse_models_nonopt(options, maximum_models=0, additional_code='', getstats=False):
  addClaspOptions = ''
  addGringoOptions = ''
  if getstats:
    addClaspOptions = ' --stats'
    addGringoOptions = ' --gstats'
  solver = asp.GringoClasp(
    clasp_options=options['clasp_options'] + addClaspOptions,
    gringo_options=options['gringo_options'] + addGringoOptions,
    clasp_bin=options['clasp_bin'],
    gringo_bin=options['gringo_bin'])
  asp_programs = options['parse_programs']
  if options['disambiguate']:
    asp_programs.append('semantic_disambiguation.asp')
  if options['normalize']:
    additional_code += 'normalize.'
  if options['use_features']:
    additional_code += 'use_features.'
  models = solver.run(
    asp_programs, additionalProgramText=additional_code,
    nmodels=maximum_models, collapseAtoms=True,collapseTerms=True)
  stats = None
  if getstats:
    stats = solver.parseClaspStats()
    stats.update(solver.parseGringoStats())
  return (models,stats)

def get_parse_models_opt(options, maximum_models=0, additional_code='', getstats=False):
  addClaspOptions = ''
  addGringoOptions = ''
  if getstats:
    addClaspOptions = ' --stats'
    addGringoOptions = ' --gstats'
  if options['normalize']:
    additional_code += 'normalize.'
  if options['use_features']:
    additional_code += 'use_features.'
  solver = asp.GringoClaspOpt(
    clasp_options=options['clasp_options'] + addClaspOptions,
    gringo_options=options['gringo_options'] + addGringoOptions,
    clasp_bin=options['clasp_bin'],
    gringo_bin=options['gringo_bin'])
  asp_programs = options['parse_programs']
  if options['disambiguate']:
    asp_programs.append('semantic_disambiguation.asp')
  models = solver.run(
    asp_programs, additionalProgramText=additional_code,
    nmodels=maximum_models, collapseAtoms=True,collapseTerms=True)
  stats = None
  if getstats:
    stats = solver.parseClaspStats()
    stats.update(solver.parseGringoStats())
  return (models,stats)

class IDPDrawConverter:
  '''
  converter from parse models to idpdraw files

  use as follows:
  IDPDrawConverter().convertParseModels2IDPDrawFile(options, models)
  
  -> output goes into options['idpdrawfile']
  '''
  def convertParseModels2IDPDrawFile(self, options, models):
    # first parse
    solver_idpdraw = asp.GringoClasp(
      clasp_bin=options['clasp_bin'],
      gringo_bin=options['gringo_bin'])
    idpdraw_models = []
    for parse_model in models:
      #debug(parse_model)
      idpdraw_localmodels = solver_idpdraw.run(
        [options['idpdraw_asp'], parse_model.to_file()],
        nmodels=2, collapseAtoms=False,collapseTerms=False)
      #debug("#models:" + str(len(idpdraw_localmodels)))
      #import subprocess
      #subprocess.Popen(["echo", "First Model:"])
      #subprocess.Popen(["cat", idpdraw_localmodels[0].to_file()])
      #if len(idpdraw_localmodels) == 2:
      #  subprocess.Popen(["echo", "Second Model:"])
      #  subprocess.Popen(["cat", idpdraw_localmodels[1].to_file()])
      if len(idpdraw_localmodels) != 1:
        raise RuntimeError("expected exactly 1 idpdraw model for parser model" +
          " got %d" % (len(idpdraw_localmodels),))
      idpdraw_models.append(idpdraw_localmodels[0])
      #debug("idpdraw model = %s" % (idpdraw_localmodels[0],))
    self.convertModelsIntoIDPDrawFile(idpdraw_models, options['idpdrawfile'])

  def formatCategoryInternal(self, term, outerBrackets=True):
    if term.__class__ != asp.Term:
      # strings are stripped of their '"'
      # integers are simply returned as strings
      return str(term).strip('"')

    args = map(lambda x: self.formatCategoryInternal(x), term.args())

    core = ''
    if len(args) == 2:
      slash = None
      if term.pred() == 'lfunc':
        slash = '\\'
      elif term.pred() == 'rfunc':
        slash = '/'
      else:
        raise RuntimeError("cannot convert term %s in formatCategoryInternal 1" % term)
      assert(slash != None)
      core = '%s%s%s' % (args[0], slash, args[1])
    else:
      if term.pred() == 'conj' and len(args) == 1:
        core = '&%s&' % args[0]
      else:
        raise RuntimeError("cannot convert term %s in formatCategoryInternal 2" % term)
    if outerBrackets:
      return '(' + core + ')'
    else:
      return core

  def formatCategory(self,term):
    return '"' + self.formatCategoryInternal(term,False) + '"'

  def convertModelsIntoIDPDrawFile(self, models, ofname):
    debug("writing %d models to idpdraw file '%s'" % (len(models),ofname,))
    #debug(models)
    overallmodel = asp.TermSet([])
    counter = 0
    for model in models:
      # first remove everything that does not start with idpd_
      for atom in model:
        if not atom.pred().startswith('idpd_'):
          continue
        atom.args().insert(0,counter)
        if atom.pred() == 'idpd_text_t':
          lastarg = atom.args()[-1]
          #debug(lastarg)
          if lastarg.__class__ == asp.Term:
            atom.args()[-1] = self.formatCategory(lastarg)
        #debug(str(atom))
        overallmodel.add(atom) 
      counter += 1
    overallmodel.to_file(ofname)

def print_interesting_stats(prefix, stats):
  #debug(stats)
  if stats == None:
    return
  def obtain(key):
    if key in stats:
      return stats[key]
    else:
      return '-'
  def obtainfloatseconds(key):
    if key in stats:
      return float(stats[key][:-1])
    else:
      return '-'
  overall = obtainfloatseconds('clasp/Time')
  if overall != '-':
    # round to two decimal places (otherwise grounding time may be negative)
    overall = round(overall,2)
  firstmodel = obtainfloatseconds('clasp/Time/1st Model')
  solving = obtainfloatseconds('clasp/Time/Solving')
  unsat = obtainfloatseconds('clasp/Time/Unsat')
  grounding = '-'
  if overall != '-' and firstmodel != '-' and solving != '-' and unsat != '-':
    grounding = overall-solving #-firstmodel-unsat
  debug(('statistics %s: overall %s grounding %s firstmodel %s solving %s unsat %s choices %s conflicts %s '+
        'restarts %s equivalences %s variables %s constraints %s lemmas %s') % \
    (prefix, overall, str(grounding), firstmodel, solving, unsat,
    obtain('clasp/Choices'), obtain('clasp/Conflicts'), obtain('clasp/Restarts'),
    obtain('clasp/Equivalences'), obtain('clasp/Variables'), obtain('clasp/Constraints'), obtain('clasp/Lemmas')))

def launchIDPDraw(options):
  import subprocess
  proc = subprocess.Popen([options['idpdraw_bin'], options['idpdrawfile']])
  proc.wait()
