# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------

from bioasp.asp import *
from bioasp.misc import *

global optimize
# use 0 for debugging
# use 1 for speed (run python once without -O or -OO and then with -O or -OO)
optimize=1
import sys
if sys.flags.optimize in ['-O', '-OO'] and optimize == 0:
  raise RuntimeError("need to use optimize=1 and run python without -O or -OO to make parsers work")

class Lexer:
	tokens = (
		'IDENT',
		'ARROW',
		'PLUS',
		'MINUS',
		'UND',
		'COMPL',
		'OR',
		'LP',
		'RP',
		'ENDL',
		'COM',
	)

	# Tokens

	t_IDENT = r'[a-zA-Z][a-zA-Z0-9_:\-\[\]/]*'
	t_ARROW = r'->'
	t_PLUS = r'\+'
	t_MINUS = r'-'
	t_UND = r'\?'
	t_COMPL = r'&'
	t_OR = r'\|'
	t_LP = r'\('
	t_RP = r'\)'
	t_ENDL = r'\n'
	t_COM = r','

	#t_annulationc = r'annulation_complex'
	#t_competition = r'competition_i'
	#t_my_or       = r'my_or'
	#t_strong_inhibitor = r'strong_inhibitor'
	#t_many_ors = r'many_ors'
	#t_not_and = r'not_and'

	#t_depends_on_sig = r'depends_on_sig'


	def __init__(self):
		import bioasp.ply.lex as lex
		global optimize
		self.lexer = lex.lex(object=self, optimize=optimize, lextab='calc_lextab')

	# Ignored characters
	t_ignore = " \t"

	def t_newline(self, t):
		r'\n+'
		t.lexer.lineno += t.value.count("\n")
		
	def t_error(self, t):
		print "Illegal character '%s'" % t.value[0]
		t.lexer.skip(1)


class Parser:
	tokens = Lexer.tokens

	precedence = ( )
		#('left','PLUS','MINUS'),
		#('left','TIMES','DIVIDE'),
		#('right','UMINUS'),
		#)

	def __init__(self):
		# dictionary of names
		self.aux_node_counter=0
		#self.names = { }
		self.accu = TermSet()
		self.fun_flag = False
		self.args = []
		self.commas = []
		self.lexer = Lexer()
		import bioasp.ply.yacc as yacc
		global optimize
		self.parser = yacc.yacc(module=self, optimize=optimize, tabmodule='calc_parsetab', debugfile="calc_parser.out")
		#self.parser = yacc.yacc(module=self)

	def p_statement_expr(self, t):
		'''statement : node_expression ARROW node_expression value
					| node_expression ARROW value
					| node_expression '''
		if len(t) == 2: 
			self.accu.add(Term('input', [t[1]]))
		elif len(t) == 4:
			print "warning: ignoring line 'node_expression ARROW value' with values '%s %s %s'" % (t[1],t[2],t[3])
		elif len(t) == 5:
			self.accu.add(Term('edge', [t[1],t[3]]))
			if t[4]!="0" : self.accu.add(Term('obs_elabel', [t[1],t[3],t[4]]))
			#self.names[t[0]] = t[2]

	def p_node_expression(self, t):
		'''node_expression : IDENT 
					  | IDENT LP commalist RP 
					  | IDENT LP node_expression RP
					  | orlist2
					  | andlist2'''
		if len(t)<3 : 
			
			if self.fun_flag==True :
				t[0]=t[1]
				self.fun_flag=False
			else :
				t[0] = "gen(\""+t[1]+"\")"
				self.accu.add(Term('vertex', ["gen(\""+t[1]+"\")"]))
		elif t[1] == 'opposite_sign' :
			self.aux_node_counter+=1
			aux_node = "aux_node_"+str(self.aux_node_counter)
			t[0] = "opposite_sign"+"("+aux_node+")"
			self.accu.add(Term('edge', [t[3],t[1]+"("+aux_node+")"]))
			self.accu.add(Term('obs_elabel', [t[3],t[1]+"("+aux_node+")","1"]))
		elif t[1] == 'strong_inhibitor' :
			t[0] = "strong_inhibitor"+"("+t[3]+")"
			for i in self.commas : 
				self.accu.add(Term('edge', [i,t[1]+"("+t[3]+")"]))
				self.accu.add(Term('obs_elabel', [i,t[1]+"("+t[3]+")","1"]))
			self.commas= []
		else : t[0] = "unknown"

	def p_value(self, t):
		'''value : PLUS
		       | MINUS
		       | UND '''
		if t[1] == '-' : t[0] = "-1"
		elif t[1] == '+' : t[0] = "1"
		elif t[1] == '?' : t[0] = "0"

	def p_commalist(self, t):
		'''commalist : IDENT COM IDENT'''
					  #| commalist COM IDENT'''
		#t[0] = t[1]+" , "+t[3]
		t[0] = "gen(\""+t[1]+"\"),gen(\""+t[3]+"\")"
		self.accu.add(Term('vertex', ["gen(\""+t[1]+"\")"]))
		self.accu.add(Term('vertex', ["gen(\""+t[3]+"\")"]))
		self.commas.append("gen(\""+t[1]+"\")")
		self.commas.append("gen(\""+t[3]+"\")")

	def p_orexpression2(self, t):
		'orlist2 : orlist'
		self.aux_node_counter+=1
		aux_node = "aux_node_"+str(self.aux_node_counter)
		self.accu.add(Term('vertex', ["or("+aux_node+")"]))
		for i in self.args : 
			self.accu.add(Term('edge', [i,"or("+aux_node+")"]))
			self.accu.add(Term('obs_elabel', [i,"or("+aux_node+")","1"]))
		self.args= []
		t[0]="or("+aux_node+")"
		
	def p_andexpression2(self, t):
		'andlist2 : andlist'
		self.aux_node_counter+=1
		aux_node = "aux_node_"+str(self.aux_node_counter)
		self.accu.add(Term('vertex', ["and("+aux_node+")"]))
		for i in self.args : 
			self.accu.add(Term('edge', [i,"and("+aux_node+")"]))
			self.accu.add(Term('obs_elabel', [i,"and("+aux_node+")","1"]))
		self.args= []
		t[0]="and("+aux_node+")"

	def p_orexpression(self, t):
		'''orlist : IDENT
				| orlist OR IDENT'''
		self.fun_flag=True
		if len(t)<3 : 
			self.accu.add(Term('vertex', ["gen(\""+t[1]+"\")"]))
			self.args.append("gen(\""+t[1]+"\")")
			t[0] = "gen(\""+t[1]+"\")"
		elif t[2] == '|' : 
			self.accu.add(Term('vertex', ["gen(\""+t[3]+"\")"]))
			self.args.append("gen(\""+t[3]+"\")")
			t[0] = "Unknown"

	def p_andexpression(self, t):
		'''andlist : IDENT
					  | andlist COMPL IDENT'''
		self.fun_flag=True
		if len(t)<3 : 
			self.accu.add(Term('vertex', ["gen(\""+t[1]+"\")"]))
			self.args.append("gen(\""+t[1]+"\")")
			t[0] = "gen(\""+t[1]+"\")"
		elif t[2] == '&' : 
			self.accu.add(Term('vertex', ["gen(\""+t[3]+"\")"]))
			self.args.append("gen(\""+t[3]+"\")")
			t[0] = "Unknown"
			
	def p_error(self, t):
		print "Syntax error at '%s'" % t
		#import inspect
		#print ''.join(map(lambda x: "  %s:%s\n    %s" % (x[1], x[2], x[4][0]),inspect.stack()))

	def parse(self, line):
		self.parser.parse(line, lexer=self.lexer.lexer)
		return self.accu
