%%%%
%%%% drawing derivation using IDPDraw (modulo postprocessing of categories)
%%%%

% we draw the word position on x axis
% we draw the time on y axis

#const wordwidth = 8.
#const poswidth = 9.
#const firsttimeheight = 1.
#const timeheight = 2.
#const catheight = 1.
#const actheight = 1.

% object names are tuples
% (first constant is main identifier)
%
% * (aux,Ident)
%   -> auxiliary stuff (captions, ...)
%      (left top coordinate of whatever)
% * (wordpos,Pos) where position(Pos)
%   -> position of surface words
%      (left top coordinate of rectangle)
% * (catpos,Pos,Time) where position(Pos), time(T)
%   -> position of category at timepoint
%      (left top coordinate of rectangle)
% * (actionpos,FirstPos,Time) where position(FirstPos), time(T)
%   -> position of action with first argument FirstPos at timepoint T
%      (left top coordinate of trapezoid)
%      (auxiliary action_width/3: action_width(FirstPos,Time,Width))

%%%%%%%
% words
%%%%%%%

% initial x positions
%
% leftmost word
% -> position X = 0
idpd_xpos_t(wordpos,Pos,X) :-
	X = 0,
	position(Pos),
	{ adjacent(PL,Pos): position(PL) } 0.
%
% words to the right
% -> position X + poswidth
idpd_xpos_t(wordpos,Pos,X+poswidth) :-
	idpd_xpos_t(wordpos,PosLeft,X),
	adjacent(PosLeft,Pos).

% word positions: Y = 0
idpd_ypos_t(wordpos,Pos,Y) :-
	Y = 0,
	position(Pos).

% word text
idpd_text_t(wordpos,P,Word) :- word_at(Word,P).
idpd_polygon_t(4,wordpos,P,0,0,wordwidth,0,wordwidth,1,0,1) :- idpd_text_t(wordpos,P,Text).

%%%%%%%%%%%%%%%%%
% overall polygon (deactivated)
%%%%%%%%%%%%%%%%%

%idpd_xpos_t(aux,overallpoly,0).
%idpd_ypos_t(aux,overallpoly,0).
%idpd_polygon_t(4,aux,overallpoly,0,0,20,0,20,30,30,0).

%%%%%%%%%%%%
% categories
%%%%%%%%%%%%

% x positions exactly where words are
idpd_xpos_t(catpos,P,T,X) :-
	idpd_xpos_t(wordpos,P,X),
	time(T).

% y positions according to time
idpd_ypos_t(catpos,P,0,firsttimeheight) :-
	position(P).
idpd_ypos_t(catpos,P,T+1,Y+timeheight) :-
	idpd_ypos_t(catpos,P,T,Y),
	time(T+1).

% category text
idpd_text_t(catpos,P,T,Text) :-
	position_category(P,Text,T),
	position_status_open(P,T).
idpd_polygon_t(4,catpos,P,T,0,0,wordwidth,0,wordwidth,catheight,0,catheight) :-
	idpd_text_t(catpos,P,T,Text).

%%%%%%%%%
% actions
%%%%%%%%%

% all actions are identified by their first position argument and by T
% actions at T are applied on fluents at T-1 and create fluents at T

% we have several kinds of actions which must be drawn differently:
% actions operating on one position and closing no position (raising)
% actions operating on two positions and closing the right one
% actions operating on three positions and closing left and right ones (coordination)

%rule_fwd_raise(P,To,T):-occurs(rule_fwd_raise,P,To,T).
%rule_bwd_raise(P,To,T):-occurs(rule_bwd_raise,P,To,T).
%rule_fwd_appl(L,R,T):-occurs(rule_fwd_appl,L,R,T).
%rule_bwd_appl(L,R,T):-occurs(rule_bwd_appl,L,R,T).
%rule_fwd_comp(L,R,T):-occurs(rule_fwd_comp,L,R,T).
%rule_bwd_comp(L,R,T):-occurs(rule_bwd_comp,L,R,T).
%rule_fwd_x_comp(L,R,T):-occurs(rule_fwd_x_comp,L,R,T).
%rule_bwd_x_comp(L,R,T):-occurs(rule_bwd_x_comp,L,R,T).
%rule_bwd_x_subst(L,R,T):-occurs(rule_bwd_x_subst,L,R,T).
%rule_coordination(L,R,T):-occurs(rule_coordination,L,R,T).

% one_position_action(Position,Time,Descriptiontext)
unaryMap(rule_fwd_raise,ftr).
unaryMap(rule_bwd_raise,btr).
unaryMap(rule_unary_type_change,tc).
one_position_action(P,T+1,UnaryText) :-
	occurs(Unary,P,To,T), unary(Unary), unaryMap(Unary,UnaryText).
%one_position_action(P,T+1,ftr) :- rule_fwd_raise(P,To,T).
%one_position_action(P,T+1,btr) :- rule_bwd_raise(P,To,T).

% two_position_action(LPosition,Time,RPosition,Descriptiontext)
binaryMap(rule_fwd_appl,fa).
binaryMap(rule_bwd_appl,ba).
binaryMap(rule_fwd_comp,fc).
binaryMap(rule_bwd_comp,bc).
binaryMap(rule_bwd_x_comp,bxc).
binaryMap(rule_bwd_x_subst,bxs).
binaryMap(rule_left_comma,commal).
binaryMap(rule_left_colon,colonl).
binaryMap(rule_left_semicolon,scolonl).
binaryMap(rule_right_comma,commar).
binaryMap(rule_right_colon,colonr).
binaryMap(rule_right_semicolon,scolonr).
two_position_action(L,T+1,R,BinaryText) :-
	occurs(Binary,L,T), binary(Binary), binaryMap(Binary,BinaryText), position_adjacent(L,R,T).
%two_position_action(L,T+1,R,fa) :- rule_fwd_appl(L,R,T).
%two_position_action(L,T+1,R,ba) :- rule_bwd_appl(L,R,T).
%two_position_action(L,T+1,R,fc) :- rule_fwd_comp(L,R,T).
%two_position_action(L,T+1,R,bc) :- rule_bwd_comp(L,R,T).
%two_position_action(L,T+1,R,bxc) :- rule_bwd_x_comp(L,R,T).
%two_position_action(L,T+1,R,bxs) :- rule_bwd_x_subst(L,R,T).

% three_position_action(LPosition,Time,MPosition,RPosition,Descriptiontext)
ternaryMap(rule_coordination,conj).
three_position_action(L,T+1,M,R,TernaryText) :-
	occurs(Ternary,L,T), ternary(Ternary), ternaryMap(Ternary,TernaryText),
	position_adjacent(L,M,T), position_adjacent(M,R,T).
%three_position_action(L,T+1,M,R,conj) :- rule_coordination(L,R,T),
%	position_adjacent(L,M,T),
%	position_adjacent(M,R,T).

% action descriptions are identified by leftmost positions
action_desc(L,T,Desc) :- one_position_action(L,T,Desc).
action_desc(L,T,Desc) :- two_position_action(L,T,R,Desc).
action_desc(L,T,Desc) :- three_position_action(L,T,M,R,Desc).

% one position actions have a width of the width of the position - 2
action_width(P,T,wordwidth-2) :- one_position_action(P,T,Desc).
% one position actions are positioned to the center of the position
% -> given a width of position with - 2, they are positioned at position+1
idpd_xpos_t(actionpos,P,T,X+1) :-
	one_position_action(P,T,Desc),
	idpd_xpos_t(catpos,P,T-1,X),
	time(T-1).

% two position actions are positioned to the center of the left position
idpd_xpos_t(actionpos,L,T,X2) :-
	two_position_action(L,T,R,Desc),
	idpd_xpos_t(catpos,L,T-1,X1),
	time(T-1),
	X2 = (X1+wordwidth/2).
% two position actions have a width of the distance between their two positions
action_width(L,T,W) :- two_position_action(L,T,R,Desc),
	idpd_xpos_t(catpos,L,T-1,X1),
	idpd_xpos_t(catpos,R,T-1,X2),
	W = X2 - X1.

% three position actions are positioned to the center of the left position
idpd_xpos_t(actionpos,L,T,X2) :-
	three_position_action(L,T,M,R,Desc),
	idpd_xpos_t(catpos,L,T-1,X1),
	time(T-1),
	X2 = (X1+wordwidth/2).
% three position actions have a width of the distance between their left and right positions
action_width(L,T,W) :- three_position_action(L,T,M,R,Desc),
	idpd_xpos_t(catpos,L,T-1,X1),
	idpd_xpos_t(catpos,R,T-1,X2),
	W = X2 - X1.

% all actions have same y position below category of previous time
idpd_ypos_t(actionpos,P,T,Y+catheight) :-
	action_desc(P,T,Desc),
	idpd_ypos_t(catpos,P,T-1,Y).

% text for action
idpd_text_t(actionpos,P,T,Desc) :-
	action_desc(P,T,Desc).
% polygon for action
idpd_polygon_t(4,actionpos,P,T,0,0,Width,0,Width-1,1,1,1) :-
	action_desc(P,T,Desc),
	action_width(P,T,Width).

%#hide.
%#show idpd_xpos_t/3.
%#show idpd_xpos_t/4.
%#show idpd_ypos_t/3.
%#show idpd_ypos_t/4.
%#show idpd_text_t/3.
%#show idpd_text_t/4.
%#show idpd_color_t/5.
%#show idpd_color_t/6.
%#show idpd_pencolor_t/5.
%#show idpd_pencolor_t/6.
%#show idpd_polygon_t/11.
%#show idpd_polygon_t/12.

% vim:syn=prolog:
