% word_entry/2
%   links words in lexicon to lexicon entries
%   (convention: words are prefixed with "s")
%   (convention: lexicon entries are words with a number suffix)
% entry_category/2
%   gives one (possibly complex) category for each lexicon entry
%   categories are given in functional style:
%   rfunc(C1,C2) denotes a CCG functor "C1/C2" which
%     has type C1 and takes an argument C2 to the right
%   lfunc(C1,C2) denotes a CCG functor "C1\C2" which
%     has type C1 and takes an argument C2 to the left
%   basic categories are prefixed with "c"

% determiner: NP/N
% the, a
word_entry("the",determiner).
word_entry("a",determiner).
entry_category(determiner,rfunc("NP","N")).

% and/or: Conj
word_entry("and",conjunction1).
word_entry("or",conjunction1).
entry_category(conjunction1,"Conj").

% common nouns

% nouns: N
% cat, dog, man, book
word_entry("cat",noun).
word_entry("dog",noun).
word_entry("man",noun).
word_entry("book",noun).
entry_category(noun,"N").

% proper nouns

% John: NP
word_entry("John",john1).
entry_category(john1,"NP").

% Ed: NP
word_entry("Ed",ed1).
entry_category(ed1,"NP").

% Ann: NP
word_entry("Ann",ann1).
entry_category(ann1,"NP").

% Marcel: NP
word_entry("Marcel",marcel1).
entry_category(marcel1,"NP").

% Chicago: NP
word_entry("Chicago",chicago1).
entry_category(chicago1,"NP").

% Monday: NP
word_entry("Monday",monday1).
entry_category(monday1,"NP").

% "other"

% I: NP
word_entry("I",i1).
entry_category(i1,"NP").

% completeness: NP
word_entry("completeness",completeness1).
entry_category(completeness1,"NP").

% everyone: NP
word_entry("everyone",everyone1).
entry_category(everyone1,"NP").

% someone: NP
word_entry("someone",someone1).
entry_category(someone1,"NP").

% to: ((S\NP)\(S\NP))/NP
% on: ((S\NP)\(S\NP))/NP
% in: ((S\NP)\(S\NP))/NP
word_entry("to",preposition).
word_entry("on",preposition).
word_entry("in",preposition).
entry_category(preposition,rfunc(lfunc(lfunc("S","NP"),lfunc("S","NP")),"NP")).

% auxiliaries

% will: (S/VP)\NP = (S/(S\NP))\NP
word_entry("will",aux1).
entry_category(aux1,lfunc(rfunc("S",lfunc("S","NP")),"NP")).

% verbs

% went: (S\NP) "intransitive"
word_entry("went",went1).
entry_category(went1,lfunc("S","NP")).

% ran: (S\NP) "intransitive"
word_entry("ran",ran1).
entry_category(ran1,lfunc("S","NP")).

% walks: (S\NP) "intransitive"
word_entry("walks",walk1).
entry_category(walk1,lfunc("S","NP")).

% "transitive verb" (S\NP)/NP 
% walks, bit, saw, proved, loves, file
word_entry("walks",transitiveverb).
word_entry("bit",transitiveverb).
word_entry("saw",transitiveverb).
word_entry("proved",transitiveverb).
word_entry("loves",transitiveverb).
word_entry("file",transitiveverb).
entry_category(transitiveverb,rfunc(lfunc("S","NP"),"NP")).

% gave: ((S\NP)/NP)/NP "ditransitive"
word_entry("gave",gave1).
entry_category(gave1,rfunc(rfunc(lfunc("S","NP"),"NP"),"NP")).

% "transitive infinitive verb" VPing/NP
word_entry("reading",transitiveinfinitiveverb).
entry_category(transitiveinfinitiveverb,rfunc("VPing","NP")).

% adverbs

% today: S\S "sentence-modifying"
word_entry("today",today1).
entry_category(today1,lfunc("S","S")).

% yesterday: S\S "sentence-modifying"
word_entry("yesterday",yesterday1).
entry_category(yesterday1,lfunc("S","S")).

% always: (S\NP)/(S\NP) "verb-modifying to the right"
% TODO: is this only intransitive-verb-modifying?
word_entry("always",rverbmodifying).
entry_category(rverbmodifying,rfunc(lfunc("S","NP"),lfunc("S","NP"))).

% today: (S\NP)\(S\NP) "verb-modifying to the left"
% yesterday: same
% TODO: is this only intransitive-verb-modifying?
word_entry("today",lverbmodifying).
word_entry("yesterday",lverbmodifying).
entry_category(lverbmodifying,lfunc(lfunc("S","NP"),lfunc("S","NP"))).

% adverbial phrase with infinitive: (VP\VP)/VPing = ((S\NP)\(S\NP))/VPing
% by <...ing>, without <...ing>
word_entry("by",advinf).
word_entry("without",advinf).
entry_category(advinf,rfunc(lfunc(lfunc("S","NP"),lfunc("S","NP")),"VPing")).

