"""
ASPCCGTK parse_and_display

Copyright (C) 2011, 2012, 2013 Yuliya Lierler
Copyright (C) 2011, 2012, 2013 Peter Schueller

requires python >= 2.6
"""

from aspccgtk import *

import sys
from bioasp import asp
import time

options = {}
options['maximum_models'] = 50
options['dictionary_asp']='/dev/null'
#options['parser_asp']='ccg.asp'
options['parser_asp']='cykccg.asp'
#options['idpdraw_asp'] = 'ccg2idpdraw.asp'
options['idpdraw_asp'] = 'cykccg2idpdraw.asp'
options['maxstep_options'] = '--const maxsteps=8'
options['numtr_options'] = '--const numtr=2'
options['clasp_bin'] = './claspwrapper.sh'
options['gringo_bin'] = './gringowrapper.sh'
options['idpdraw_bin'] = './IDPDraw/code/IDPDraw'
options['no_display'] = False
options['normalize'] = True
options['only_strict'] = False
options['stats'] = False
options['use_features'] = False

# activate semantic disambiguation here (preliminary, for spaghetti and meatballs example)
options['disambiguate'] = False

process_commandline(options)

# do some default input
if 'input_asp' not in options:
  options['dictionary_asp']='common_dict.asp'
  #options['input_asp']='../testsuite/instances/john_saw_the_cat_and_the_dog/input'
  options['input_asp']='../testsuite/instances/john_saw_the_cat_in_chicago_and_the_dog_on_monday/input'
  #options['input_asp']='examples_common_dict/the_dog_bit_john.asp'

if options['dictionary_asp']=='/dev/null':
  debug("No DICTIONARY is Given")

options['clasp_options'] = ''
options['gringo_options'] = options['maxstep_options']+" "+options['numtr_options']
options['idpdrawfile'] = options['input_asp'] + '.idpdraw'
options['parse_programs'] = [ options['parser_asp'], options['input_asp'], options['dictionary_asp'] ]

debug("Parser: "+options['parser_asp'])
debug("Gringo Options: "+options['maxstep_options'])

analysis = analyze_instance(options)
debug('got analysis infos ' + repr(analysis))
maxprobclass = analysis['maxprobclass']

timeStart = time.time()

probability_class_additionalcode = ''

# first try to parse with strict result
# if we have probability classes, try all starting with smallest
if maxprobclass == None:
  # parse strict without probability class
  debug("parsing with strict goal ...")
  parse_additional = 'strict_parse.'
  (parse_models, stats) = get_parse_models_nonopt(
    options, options['maximum_models'], parse_additional, options['stats'])
  print_interesting_stats('strict_noprob', stats)
  #debug("strict parse models = %s" % (parse_models,))
else:
  # try to increase probability classes until we find at least one parse
  for cls in range(0,maxprobclass+1):
    debug("parsing with strict goal and probability class 0-%d..." % (cls,))
    probability_class_additionalcode = 'use_category_class(0..%d).' % (cls,)
    parse_additional = 'strict_parse. ' + probability_class_additionalcode
    (parse_models, stats) = get_parse_models_nonopt(
      options, options['maximum_models'], parse_additional, options['stats'])
    print_interesting_stats('strict_prob%d' % (cls,), stats)
    debug("strict parse models = %s" % (parse_models,))
    if len(parse_models) != 0:
      break

timeStrict = time.time()
debug("TIME strict %0.3f" % (timeStrict-timeStart,))

if len(parse_models) == 0 and not options['only_strict']:
  debug("parsing with optimization ...")
  optparse_additional = '#minimize { minimizationCriterion(X,Y) }.' + probability_class_additionalcode
  (optparse_models, stats) = get_parse_models_opt(
    options, options['maximum_models'], optparse_additional, options['stats'])
  print_interesting_stats('opt', stats)
  #debug("optimization parse models = %s" % (optparse_models,))
  if optparse_models == None:
    raise RuntimeError("unsatisfiable even with optimization!")
  parse_models = optparse_models[1]
  timeOpt = time.time()
  debug("TIME opt %0.3f total %0.3f" % (timeOpt-timeStrict, timeOpt-timeStart))
#debug("parse models (strict or opt): %s" % parse_models)

debug("got %d parse models (maximum set to %d)" % (len(parse_models), options['maximum_models']))

if not options['no_display']:
  debug("calculating idpdraw answer sets")
  IDPDrawConverter().convertParseModels2IDPDrawFile(options, parse_models)
  # display with idpdraw
  launchIDPDraw(options)
