frame(food).
frame_element(food,food_constituent).

frame(ingestion).
frame_element(ingestion,ingestible).
frame_element(ingestion,ingestor).
frame_element(ingestion,instrument).


is_a(sentient,ingestor).
is_a(food,ingestible).


instance_of("chopsticks",instrument).
instance_of("spaghetti",food).
instance_of("meatballs",food_constituent).
instance_of("eat",ingestion).
instance_of("I",sentient).

% what's a semtype
semtype(X) :- instance_of(_,X).
semtype(X) :- is_a(X,_).
semtype(X) :- is_a(_,X).

% every semtype is its own semtype
is_a(X,X) :- semtype(X).
% is_a is transitive
is_a(X,Z) :- is_a(X,Y), is_a(Y,Z).

%
% after selecting the parse tree,
% each grid cell may have only one semtype but several features
%

%
% lexical items at P,P define their semtype
%
% TODO future work: ambiguous semtype, unknown-semtype heuristics
grid_semtype(P,P,Sem) :- word_at(W,P), instance_of(W,Sem).

% frames define unused frame elements at the word level
grid_frame_element(P,P,Element) :-
   grid_semtype(P,P,Frame), frame(Frame), frame_element(Frame,Element).


% now define:
% when we compose two things:
% do we have a frame that uses a semtype of the other semantic object?
% (this is like a feature check which "uses up" both the feature and the check)

% TODO make this more elegant, map from CYK grid to tree representation
% and define semantic disambiguation on that tree representation

% use frame element from frame in downwards gridfield by left gridfield
2 { grid_frame_element_used(Pj,Pdown,Sem),
    grid_semtype_used(Pleft,Pi,OSem) } 2 :-
  occurs_grid(_,Pj,Pi,Pleft,Pdown,_,_,_),
  % leftwards object
  grid_semtype(Pleft,Pi,OSem), 
  is_a(OSem,Sem),
  % downwards frame
  grid_frame_element(Pj,Pdown,Sem).
% if frame_element uses the semtype, we hide all grid semtypes of the other field
%grid_semtype_hidden(Pleft,Pi,OSem) :-
%  occurs_grid(_,Pj,Pi,Pleft,Pdown,_,_,_),
%  grid_frame_element_used(Pj,Pdown,_),
%  grid_semtype(Pleft,Pi,OSem).
% if frame_element uses the semtype, we hide all frame elements of the other field
grid_frame_element_hidden(Pleft,Pi,Element) :-
  occurs_grid(_,Pj,Pi,Pleft,Pdown,_,_,_),
  grid_frame_element_used(Pj,Pdown,_),
  grid_frame_element(Pleft,Pi,Element).

% use frame element from frame in leftwards gridfield by downwards gridfield
2 { grid_frame_element_used(Pleft,Pi,Sem),
    grid_semtype_used(Pj,Pdown,OSem) } 2 :-
  occurs_grid(_,Pj,Pi,Pleft,Pdown,_,_,_),
  % downwards object
  grid_semtype(Pj,Pdown,OSem), 
  is_a(OSem,Sem),
  % leftwards frame
  grid_frame_element(Pleft,Pi,Sem).
% if frame_element uses the semtype, we hide all grid semtypes of the other field
%grid_semtype_hidden(Pj,Pdown,OSem) :- 
%  occurs_grid(_,Pj,Pi,Pleft,Pdown,_,_,_),
%  grid_frame_element_used(Pleft,Pi,_),
%  grid_semtype(Pj,Pdown,OSem).
% if frame_element uses the semtype, we hide all frame elements of the other field
grid_frame_element_hidden(Pj,Pdown,Element) :- 
  occurs_grid(_,Pj,Pi,Pleft,Pdown,_,_,_),
  grid_frame_element_used(Pleft,Pi,_),
  grid_frame_element(Pj,Pdown,Element).

grid_frame_element_hidden(Pi,Pj,Element) :- grid_frame_element_used(Pi,Pj,Element).
grid_semtype_hidden(Pi,Pj,Sem) :- grid_semtype_used(Pi,Pj,Sem).


% those semtypes that are not hidden accumulate
grid_semtype(Pj,Pi,Sem) :-
  occurs_grid(_,Pj,Pi,Pleft,Pdown,_,_,_),
  grid_semtype(Pleft,Pi,Sem),
  not grid_semtype_hidden(Pleft,Pi,Sem). 
grid_semtype(Pj,Pi,Sem) :-
  occurs_grid(_,Pj,Pi,Pleft,Pdown,_,_,_),
  grid_semtype(Pj,Pdown,Sem),
  not grid_semtype_hidden(Pj,Pdown,Sem). 

% those grid_frame_elements that are not hidden accumulate
grid_frame_element(Pj,Pi,Sem) :-
  occurs_grid(_,Pj,Pi,Pleft,Pdown,_,_,_),
  grid_frame_element(Pleft,Pi,Sem),
  not grid_frame_element_hidden(Pleft,Pi,Sem). 
grid_frame_element(Pj,Pi,Sem) :-
  occurs_grid(_,Pj,Pi,Pleft,Pdown,_,_,_),
  grid_frame_element(Pj,Pdown,Sem),
  not grid_frame_element_hidden(Pj,Pdown,Sem). 


%root_frame_elements(Pj,Pi,Sem) :- grid_frame_element(Pj,Pi,Sem).%, Pj=4,Pi=1.
%root_semtype(Pj,Pi,Sem) :- grid_semtype(Pj,Pi,Sem).%, Pj=4,Pi=1.
%#show grid_frame_element/3.
%#show grid_frame_element_used/3.
%#show grid_semtype/3.
%#show grid_semtype_used/3.

% define what's a bad state in the tree:
bad_semtype(Pi,Pj,Sem1,Sem2) :- grid_semtype(Pi,Pj,Sem1), grid_semtype(Pi,Pj,Sem2), Sem1 != Sem2.
%#show bad_semtype/4.
:- bad_semtype(_,_,_,_).
